    processor 6502
    include vcs.h

;TODOs:
; - autodetect Kool Aid Man bug


;===============================================================================
; A S S E M B L E R - S W I T C H E S
;===============================================================================

NTSC            = 1
KOOL_AID_OK     = 1     ; disable if console has problems with Kool Aid Man
STATIC          = 1     ; currently doesn't work!

SC_LOAD         = 0


;===============================================================================
; C O N S T A N T S
;===============================================================================

INVADER_H       = 9
MISSILE_H       = 6

ROW_H           = INVADER_H+6

NUM_POS_P0      = 6
NUM_POS_P1      = 5
NUM_POSITIONS   = NUM_POS_P0+NUM_POS_P1

NUM_ROWS        = 5

RAND_EOR_8      = $b2

DX              = 0


  IF NTSC
RED             = $40
BLUE            = $80
  ELSE
RED             = $60
BLUE            = $b0
  ENDIF

op_nop_imm      = $80
op_nop_zp       = $04
op_nop_abs      = $0c
op_nop          = $ea

op_bit          = $24
op_sta          = $85
op_lda_zpx      = $b5
op_sta_zpx      = $95

DUMMY           = $2d


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

RAMKernel   ds $30  ; KernelCodeEnd - KernelCode (48 bytes reserved for RAM kernel)

ctrlSC      = $80

tmpVar      .ds 4

frameCnt    .byte
random      .byte
reload      .byte

position    .byte
walkDir     .byte
yMsl0       .byte

invPat0     .ds NUM_ROWS
invPat1     .ds NUM_ROWS

xPos        .byte


;===============================================================================
; M A C R O S
;===============================================================================

  MAC DEBUG_BRK
    IF DEBUG
      brk                         ;
    ENDIF
  ENDM

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM

  MAC CHECKPAGE
    IF >. != >{1}
      ECHO ""
      ECHO "ERROR: different pages! (", {1}, ",", ., ")"
      ECHO ""
      ERR
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank0
    ORG     $f000

;---------------------------------------------------------------
KernelCode SUBROUTINE       ;               patched into
.loop:                      ;       @61..71
PatchInv:
    lda     InvaderTbl-1,y  ; 4
PatchHM:
    sta     HMOVE           ; 3     @68..   bit DUMMY           early HMOVEs!
Patch0:
    sta     RESP0           ; 3     @71..   bit DUMMY           1st invader A
Patch1:
    sta     RESP1           ; 3     @74..   bit DUMMY           2nd invader A
PatchGRP0:
    sta     GRP0            ; 3             lda GRP0
PatchGRP1:
    sta     GRP1            ; 3             lda GRP1
PatchMsl:
    lda     ENABLTbl-1,y    ; 4
    sta     ENABL -DX,x     ; 4                                 waste one cycle
PatchRESP0:
Patch2:
    sta     RESP0 -DX,x     ; 4     @16..   bit DUMMY           1st invader B, waste one cycle
PatchRESP1:
Patch3:
    sta     RESP1 -DX,x     ; 4     @20..   lda DUMMY -DX,x     2nd invader B
    sta     RESP0 -DX,x     ; 4     @24..   lda RESP0 -DX,x
    sta     RESP1 -DX,x     ; 4             lda RESP1 -DX,x
    sta     RESP0 -DX,x     ; 4             lda RESP0 -DX,x
    sta     RESP1 -DX,x     ; 4             lda RESP1 -DX,x
    sta     RESP0 -DX,x     ; 4             lda RESP0 -DX,x
    sta     RESP1 -DX,x     ; 4             lda RESP1 -DX,x
    sta     RESP0 -DX,x     ; 4             lda RESP0 -DX,x
    sta     RESP1 -DX,x     ; 4             lda RESP1 -DX,x
    sta     RESP0 -DX,x     ; 4     @56..   lda RESP0 -DX,x
    dey                     ; 2
    bne     .loop           ; 2
    jmp     ContKernel      ; 3     @63..
KernelCodeEnd:

PD = KernelCode - RAMKernel


Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    inc     random

    ldx     #KernelCodeEnd-KernelCode-1
.loopCopy:
    lda     KernelCode,x
    sta     RAMKernel,x
    dex
    bpl     .loopCopy

    lda     #1
    sta     CTRLPF

    lda     #%111111
;    lda     #0
;    lda     #%110000
    sta     invPat0
    lda     #%11111
;    lda     #1
    sta     invPat1

    sta     WSYNC
    SLEEP   45
    sta     RESBL
    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
  IF NTSC
    ldx     #44
  ELSE
    ldx     #44+18
  ENDIF
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


GameCalc SUBROUTINE
    inc     frameCnt

    lda     frameCnt
    and     #$1f
    bne     .skipMove
    ldx     position
    bit     walkDir
    bmi     .negDir

    inx
    cpx     #NUM_POSITIONS
    bne     .setPos
    dex
    dex
    bne     .invDir

.negDir:
    dex
    bpl     .setPos
    inx
    inx
.invDir:
    lda     walkDir
    eor     #$80
    sta     walkDir
.setPos
    stx     position

.skipMove:
    lda     #BLUE|$6
    sta     COLUPF
    ldy     #$0c
    ldx     #$0c
    lda     #0
    bit     SWCHB
    bmi     .skipDemo
    lda     #$02
    sta     COLUPF
    ldy     #RED|$a
    ldx     #BLUE|$a
    lda     #$55
.skipDemo:
    sty     COLUP0
    stx     COLUP1
    sta     PF0
    sta     PF2
    asl
    sta     PF1

    rts


    align   256

DrawScreen SUBROUTINE
.row        = tmpVar
.yMsl       = tmpVar+1
.jmpVec     = tmpVar+2      ;..3
.tmpHMP0    = Patch0+1 - PD

    lda     yMsl0
    sta     .yMsl
    lda     #>Delays
    sta     .jmpVec+1

  IF NTSC
    ldx     #227+5
  ELSE
    ldx     #227+27
  ENDIF
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
;---------------------------------------
    sta     VBLANK
    stx     TIM64T

  IF STATIC
;    ldx     #0
    ldx     #NUM_POSITIONS-1
  ELSE
    ldx     #NUM_ROWS-1
  ENDIF
.loopColumns:                       ;           @07..17
    stx     .row                    ; 3 =  3

; ***** patch kernel positioning: *****
  IF STATIC
    ldy     .row                    ; 3
  ELSE
    ldy     position                ; 3
  ENDIF
    sta     HMCLR                   ; 3             stop ball from moving
    lda     HMP0Tbl,y               ; 4
    sta     .tmpHMP0                ; 3
    lda     HMP1Tbl,y               ; 4
    sta     HMP1                    ; 3 = 17

; ***** position missile: *****
    lda     .yMsl
    sec
    sbc     #ROW_H
    sta     .yMsl
    sec
    sbc     #INVADER_H+MISSILE_H
    bcs     .noMissile
    adc     #<ENABLTbl+INVADER_H+MISSILE_H+2
    BIT_W
.noMissile:
    lda     #<ENABLTbl+INVADER_H+MISSILE_H
    sta     <[PatchMsl+1 - PD]

; ***** patch kernel for GRP0: *****
    ldy     #op_bit                 ; 2
    lda     invPat0                 ; 3
    beq     .hide0                  ; 2
    ldy     #op_sta                 ; 2
.hide0:
    sta     WSYNC                   ; 3 = 11/12
;---------------------------------------
    sty     PatchGRP0 - PD          ; 3

; how many bits are set (0,1,>=2)?
    beq     .empty0                 ; 2=  2/3

    ldy     #NUM_POS_P0             ; 2
.loopSingle0:
    dey                             ; 2
    lsr                             ; 2
    bcc     .loopSingle0            ; 2
    bne     .notSingle0             ; 2= 10..45

    sta     NUSIZ0                  ; 3
    sta     HMBL                    ; 3
    lda     .row                    ; 3
    asl                             ; 2
    adc     .row                    ; 3
    adc     Mult24Tbl,y             ; 4

;    ldx     #0                      ; 2
;    jsr     XPosObject              ; 6

    sta     WSYNC                   ; 3 = 23
;---------------------------------------
    sec                             ; 2
.wait0:
    sbc     #15                     ; 2
    bcs     .wait0                  ; 2
    eor     #$07                    ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    sta     HMP0                    ; 3
    sta.w   RESP0                   ; 4         @23 + 5*n!
    sta     WSYNC                   ; 3
;---------------------------------------
    sta     HMOVE                   ; 3

    ldy     #op_bit                 ; 2
    sty     <[Patch0 - PD]          ; 3
    ldy     #op_lda_zpx             ; 2
    sty     <[PatchRESP0    - PD]   ; 3
    sty     <[PatchRESP0+4  - PD]   ; 3
    sty     <[PatchRESP0+8  - PD]   ; 3
    sty     <[PatchRESP0+12 - PD]   ; 3
    sty     <[PatchRESP0+16 - PD]   ; 3
    sty     <[PatchRESP0+20 - PD]   ; 3
    lda     #$80                    ; 2
    sta     .tmpHMP0                ; 3
    bne     .endSingle0             ; 3 = 36    @36
;=======================================

.empty0:
.notSingle0:                        ;           @06,,44
    lda     invPat0                 ; 3
    ldx     #NUM_POS_P0*4-3         ; 2
.loopPat0:
    lsr                             ; 2
    dex                             ; 2
    bne     .notFirst0              ; 2
    ldy     #op_sta                 ; 2
    bcs     .patch0_1st             ; 2
    ldy     #op_bit                 ; 2
.patch0_1st:
    sty     <[Patch0 - PD]          ; 3
.notFirst0:
    ldy     #op_sta_zpx             ; 2
    bcs     .patch0                 ; 2
    ldy     #op_lda_zpx             ; 2
.patch0:
    sty     <[PatchRESP0 - PD],x    ; 4
    dex                             ; 2
    dex                             ; 2
    dex                             ; 2
    bpl     .loopPat0               ; 2= 159..168  (44+165=@57)

    lda     #%01                    ; 2
    sta     NUSIZ0                  ; 3 =  5
.endSingle0:

; ***** patch kernel for GRP1: *****
    ldy     #op_bit                 ; 2
    lda     invPat1                 ; 3
    beq     .hide1                  ; 2
    ldy     #op_sta                 ; 2
.hide1:
    sta     WSYNC                   ; 3 = 11/12
;---------------------------------------
    sty     <[PatchGRP1 - PD]       ; 3

; how many bits are set (0,1,>=2)?
    beq     .empty1                 ; 2

    ldy     #NUM_POS_P1             ; 2
.loopSingle1:
    dey                             ; 2
    lsr                             ; 2
    bcc     .loopSingle1            ; 2
    bne     .notSingle1             ; 2= 10..38

    sta     HMP0                    ; 3             prevent P0 from moving
    sta     NUSIZ1                  ; 3
    lda     .row                    ; 3
    asl                             ; 2
    adc     .row                    ; 3
    adc     Mult24Tbl,y             ; 4
    adc     #12                     ; 2

;    ldx     #1
;    jsr     XPosObject

    sta     WSYNC                   ; 3 = 23
;---------------------------------------
    sec                             ; 2
.wait1:
    sbc     #15                     ; 2
    bcs     .wait1                  ; 2
    eor     #$07                    ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    sta     HMP1                    ; 3
    sta.w   RESP1                   ; 4         @23 + 5*n!
    sta     WSYNC                   ; 3
;---------------------------------------
    sta     HMOVE                   ; 3

    ldy     #op_bit                 ; 2
    sty     <[Patch1 - PD]          ; 3
    ldy     #op_lda_zpx             ; 2
    sty     <[PatchRESP1    - PD]   ; 3
    sty     <[PatchRESP1+4  - PD]   ; 3
    sty     <[PatchRESP1+8  - PD]   ; 3
    sty     <[PatchRESP1+12 - PD]   ; 3
    sty     <[PatchRESP1+16 - PD]   ; 3
    lda     #$80                    ; 2
    sta     HMP1                    ; 3
    bne     .endSingle1             ; 3 = 33    @33
;=======================================

.empty1:
.notSingle1:                        ;           @06..37
    lda     invPat1                 ; 3
    ldx     #NUM_POS_P1*4-3         ; 2
.loopPat1:
    lsr                             ; 2
    dex                             ; 2
    bne     .notFirst1              ; 2
    ldy     #op_bit                 ; 2
    bcc     .patch1_1st             ; 2
    ldy     #op_sta                 ; 2
.patch1_1st:
    sty     <[Patch1 - PD]          ; 3
.notFirst1:
    ldy     #op_lda_zpx             ; 2
    bcc     .patch1                 ; 2
    ldy     #op_sta_zpx             ; 2
.patch1:
    sty     <[PatchRESP1 - PD],x    ; 4
    dex                             ; 2
    dex                             ; 2
    dex                             ; 2
    bpl     .loopPat1               ; 2= 131..136

    lda     #%01                    ; 2
    sta     NUSIZ1                  ; 3
.endSingle1:

    lda     .tmpHMP0                ; 3
    sta     HMP0                    ; 3
    lda     #$80                    ; 2
    sta     HMBL                    ; 3 = 11

;***** animate invaders: *****
  IF STATIC
    ldy     .row                    ; 3
  ELSE
    ldy     position                ; 3
  ENDIF
    lda     position                ; 3
    lsr                             ; 2
    lda     InvaderPtrLo,y          ; 4
    bcc     .pat0                   ; 2
    lda     InvaderPtrLo+NUM_ROWS,y ; 4
.pat0:
    sta     <[PatchInv+1 - PD]      ; 3 = 18/21

    sta     WSYNC
;---------------------------------------
;***** patch kernel for positions: *****
    SLEEP   2                       ; 2
    lda     PatchHMTbl,y            ; 4
    sta     <[PatchHM+1 - PD]       ; 3
    lda     Patch0Tbl,y             ; 4
    sta     <[Patch0+1 - PD]        ; 3
    lda     Patch1Tbl,y             ; 4
    sta     <[Patch1+1 - PD]        ; 3
    lda     Patch2Tbl,y             ; 4
    sta     <[Patch2+1 - PD]        ; 3
    lda     Patch3Tbl,y             ; 4
    sta     <[Patch3+1 - PD]        ; 3 = 37

;***** delay kernel start: *****
    lda     #<Delays-1              ; 2
    sec                             ; 2
  IF STATIC
    sbc     .row                    ; 3
  ELSE
    sbc     position                ; 3
  ENDIF
    sta     .jmpVec                 ; 3
    jmp     (.jmpVec)               ; 5+2 = 17

    .byte   op_nop_imm, op_nop_imm, op_nop_imm, op_nop_imm
    .byte   op_nop_imm, op_nop_imm, op_nop_imm, op_nop_imm
    .byte   op_nop_abs, op_nop_zp,  op_nop
Delays:
    ldx     #DX                     ; 2
    ldy     #INVADER_H              ; 2
    jmp     RAMKernel               ; 3 =  7    @61..71

ContKernel:                         ;           @63..73
    sty     GRP0                    ; 3
    sty     GRP1                    ; 3
    sty     ENABL                   ; 3

    ldx     .row                    ; 3
    dex                             ; 2
    bmi     .exitKernel             ; 2
    jmp     .loopColumns            ; 3 = 19/20

.exitKernel:
; show cursor
    bit     SWCHB
    bvs     .skipCursor
    sta     WSYNC
    ldx     #0
    stx     HMBL
    stx     NUSIZ0
    ldy     xPos
    lda     Mult12Tbl,y
    jsr     XPosObject
    ldx     #$ff
    stx     GRP0
    sta     WSYNC
    sta     WSYNC
    inx
    stx     GRP0
.skipCursor:

    ldx     #2
.waitScreen:
    lda     INTIM
    bne     .waitScreen
    sta     WSYNC
    stx     VBLANK
    rts


OverScan SUBROUTINE
  IF NTSC
    lda     #36
  ELSE
    lda     #36+15
  ENDIF
    sta     TIM64T

; move missile:
    ldx     yMsl0
    bne     .skipResetMsl
    ldx     #200
.skipResetMsl:
    dex
    stx     yMsl0

    bit     SWCHB
    bvs     .doRandom

    lda     frameCnt
    and     #$0f
    bne     .skipInput
    ldx     xPos
    lda     SWCHA
    bmi     .skipLeft
    inx
    cpx     #NUM_POSITIONS
    bne     .setPos
    ldx     #0
.skipLeft:
    asl
    bmi     .skipRight
    dex
    bpl     .setPos
    ldx     #NUM_POSITIONS-1
.skipRight:
.setPos:
    stx     xPos

    bit     INPT4
    bmi     .skipInput
    txa
    lsr
    tax
    lda     Mask2Tblb,x
    bcs     .p1
    eor     invPat0
    sta     invPat0
    bcc     .skipInput

.p1:
    lsr
    eor     invPat1
    sta     invPat1
.skipInput:
    jmp     .skipSwitch

.doRandom:
; en/disable invader column:
    lda     frameCnt
    and     #$1f
    bne     .skipSwitch
    lda     frameCnt
    and     #$20
    beq     .loopSwitch1
.loopSwitch0:
    jsr     NextRandom
    and     #$07
    cmp     #$06
    bcs     .loopSwitch0
    tay
    lda     Mask2Tbl,y
    eor     invPat0
    sta     invPat0
    bcc     .skipSwitch

.loopSwitch1:
    jsr     NextRandom
    and     #$07
    cmp     #$05
    bcs     .loopSwitch1
    tay
    lda     Mask2Tbl,y
    eor     invPat1
    sta     invPat1
.skipSwitch:

  IF SC_LOAD
; SC code:
    lda     reload
    eor     $fff9
    and     #$01
    clc
    beq     .noBit
    eor     reload
    adc     #$10
.noBit:
    sta     reload
    cmp     #$60
    bcs     .reload
.skipCheck:

    lda     SWCHB
    lsr
    bcs     .skipReset
;---------------------------------------
ram     = $f000
control = $fff8
.reload:
    lda     #$0b
    sta     ctrlSC

    cmp     ram
    cmp     control     ; rom/r3
    lda     #0
    sta     $fa         ; next load
    ldx     #100
.0
    ldy     #8
.1
    dey
    bne     .1
    dex
    bne     .0
    jmp     $f800       ; rom entry
;---------------------------------------

.skipReset:
  ELSE
    lda     SWCHB
  ENDIF
    lsr
    bcs     .skipSelect
    brk

.skipSelect:
.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


;***************************************************************
NextRandom SUBROUTINE
;***************************************************************
    lda     random              ; 3
    lsr                         ; 2
    bcc     .skipEor            ; 2
    eor     #RAND_EOR_8         ; 2
.skipEor:                       ;
    sta     random              ; 3
    rts
; NextRandom


;***************************************************************
XPosObject SUBROUTINE
;***************************************************************
    sec                             ; 2
    sta     WSYNC                   ; 3 = 23
;---------------------------------------
WaitObject:
    sbc     #15                     ; 2
    bcs     WaitObject              ; 2

    CHECKPAGE WaitObject

    eor     #$07                    ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    asl                             ; 2
    sta     HMP0,x                  ; 4
    sta.wx  RESP0,x                 ; 5         @23 + 5*n!
    sta     WSYNC                   ; 3
;---------------------------------------
    sta     HMOVE                   ; 3
    rts


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $f700, 0

    .byte   0

InvaderTbl:
Invader0Pat0:
    .byte   %00100100
    .byte   %01011010
    .byte   %01100110
    .byte   %11111111
    .byte   %11011011
    .byte   %11111111
    .byte   %01111110
    .byte   %00011000
    .byte   0
Invader0Pat1:
    .byte   %10000001
    .byte   %01011010
    .byte   %01100110
    .byte   %11111111
    .byte   %11011011
    .byte   %11111111
    .byte   %01111110
    .byte   %00011000
    .byte   0

Invader1Pat0:
    .byte   %00011000
    .byte   %10100101
    .byte   %10111101
    .byte   %11111111
    .byte   %01011010
    .byte   %00111100
    .byte   %00100100
    .byte   %01000010
    .byte   0
Invader1Pat1:
    .byte   %01000010
    .byte   %00100100
    .byte   %00111100
    .byte   %01111110
    .byte   %11011011
    .byte   %10111101
    .byte   %10100101
    .byte   %01000010
    .byte   0

Invader2Pat0:
    .byte   %00100100
    .byte   %01000010
    .byte   %00100100
    .byte   %01111110
    .byte   %01011010
    .byte   %01111110
    .byte   %00111100
    .byte   %00011000
    .byte   0
Invader2Pat1:
    .byte   %01000010
    .byte   %00100100
    .byte   %00011000
    .byte   %01111110
    .byte   %01011010
    .byte   %01111110
    .byte   %00111100
    .byte   %00011000
    .byte   0

InvaderPtrLo:
    .byte   <Invader0Pat0-1, <Invader0Pat0-1, <Invader1Pat0-1, <Invader1Pat0-1, <Invader2Pat0-1
    .byte   <Invader0Pat1-1, <Invader0Pat1-1, <Invader1Pat1-1, <Invader1Pat1-1, <Invader2Pat1-1
  IF STATIC
    .byte   <Invader0Pat0-1, <Invader0Pat0-1, <Invader1Pat0-1, <Invader1Pat0-1, <Invader2Pat0-1
    .byte   <Invader0Pat1-1
  ENDIF


ENABLTbl:
    .ds INVADER_H, 0
    .ds MISSILE_H, 2
    .ds INVADER_H, 0

HMP0Tbl:
  IF KOOL_AID_OK
    .byte   $10, $00, $00, $f0, $e0
  ELSE
    .byte   $50, $30, $20, $00, $f0
  ENDIF
    .byte   $b0, $80, $80, $80, $80, $80
HMP1Tbl:
  IF KOOL_AID_OK
    .byte   $e0, $d0
  ELSE
    .byte   $f0, $e0
  ENDIF
    .byte   $80, $80, $80, $80, $80, $80, $80, $80, $80

PatchHMTbl:
    .byte   HMOVE, HMOVE, HMOVE, HMOVE, HMOVE, HMOVE
    .byte   DUMMY, DUMMY, DUMMY, DUMMY, DUMMY
Patch0Tbl:
    .byte   RESP0, RESP0, RESP0, RESP0, RESP0, RESP0
    .byte   DUMMY, DUMMY, DUMMY, DUMMY, DUMMY
Patch1Tbl:
    .byte   RESP1, RESP1
    .byte   DUMMY, DUMMY, DUMMY, DUMMY, DUMMY, DUMMY, DUMMY, DUMMY, DUMMY
Patch2Tbl:
    .byte   DUMMY, DUMMY, DUMMY, DUMMY, DUMMY, DUMMY
    .byte   RESP0, RESP0, RESP0, RESP0, RESP0
Patch3Tbl:
    .byte   DUMMY-DX, DUMMY-DX
    .byte   RESP1-DX, RESP1-DX, RESP1-DX, RESP1-DX, RESP1-DX, RESP1-DX
    .byte   RESP1-DX, RESP1-DX, RESP1-DX

Mask2Tbl:
    .byte   $01, $02, $04, $08, $10, $20

Mask2Tblb:
    .byte   $20, $10, $08, $04, $02, $01

Mult24Tbl:
    .byte   24*0+1, 24*1+1, 24*2+1, 24*3+1, 24*4+1, 24*5+1

Mult12Tbl:
    .byte   12*0, 12*1, 12*2, 12*3, 12*4, 12*5, 12*6, 12*7, 12*8, 12*9, 12*10

    .byte   " SI-11 v0.04 - (C)2003 Thomas Jentzsch "

    org $f7fc
    .word   Start
    .word   Start
